import mixture
import mixtureLinearGaussian
import scipy
import TabDataSet
import random
import os
import getopt
import sys

def usage():
    """
    Function that print usages informations for this program.
    """
    print """
Usage: mixtureTools.py [options] <input_data_file.txt>
Options:
    -h, --help               Print this help message
    -c, --cluster            Number of clusters (default 2)
    -d, --dimension          Dimension (default 3)
    -r, --replications       Replications (default 10)
    -g, --gama               Gamma parameters for sigma sample. shape:scale (default 0.3:1)
    -n, --normal             Normal parameters for sigma(LinearGaussian) sample. mu:sigma (default 0:1)
    -s, --samples            Number of samples (default 1000)
    -S, --samples-clusters   List of number of samples for each cluster (default (100/c)% for each)
                             Ex.: -S 100:20:150 (100 samples cluster 1, 20 samples cluster 2, 150 samples cluster 3)
    -o, --output             Path dir output (default data)
"""

def command_line():
    # parse command line options
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'h:c:d:r:s:S:o:p:', ['help', 'cluster', 'dimension', 'replications', 'samples', 'samples-clusters', 'output'])
    except getopt.error, err:        
        usage()
        sys.exit(2)
    
    # parameters
    c = 2                   # number of clusters
    d = 3                   # dimension
    r = 10                  # replications
    g_shape = 1.9             # gamma shape
    g_scale = 0.4           # gamma scale
    n_mu = 0                # normal mean
    n_sigma = 2             # normal sigma (standard deviation)
    s = 1000                # number of samples
    sc = None               # samples of clusters
    output = 'data'         # output
    sparse = 0.5
    
    # process options
    for o, a in opts:
        if o in ('-h', '--help'):
            usage()
            sys.exit(0)
        elif o in ('-c', '--cluster'):
            c = int(a)
        elif o in ('-d', '--dimension'):
            d = int(a)
        elif o in ('-r', '--replications'):
            r = int(a)
        elif o in ('-g', '--gamma'):
            g_params = a.split(":")
            g_shape = float(g_params[0])
            g_scale = float(g_params[1])
        elif o in ('-n', '--normal'):
            n_params = a.split(":")
            n_mu = float(n_params[0])
            n_sigma = float(n_params[1])
        elif o in ('-s', '--samples'):
            s = int(a)
        elif o in ('-S', '--samples-clusters'):
            sc = a.split(":")
        elif o in ('-o', '--output'):
            output = a
        elif o in ('-p', '--sparse'):
            sparse = float(a)
        else:
            assert False, "unhandled option"
    
    # sample-clusters
    if sc == None:
        sc = [s / c] * c
        if (s / c) * c != s:
            sc[-1] = sc[-1] + s - (s/c)*c
    else:
        assert len(sc) == c
        for j, scj in enumerate(sc):
            sc[j] = int(scj)
    
    # create output
    try:
        os.stat(output)
    except:
        os.mkdir(output)
    
    return [c, d, r, g_shape, g_scale, n_mu, n_sigma, s, sc, sparse, output]

def main():
    c, d, r, g_shape, g_scale, n_mu, n_sigma, s, sc, sparse, output = command_line()
    
    for n in range(r):
        dists = []
        lists = []
        listdata = []
        notes = []
        gen = None
        
        for j in range(c):
            sigma = [scipy.stats.gamma.rvs(g_shape,scale=g_scale)*2]
            sigma = [random.uniform(0.2,0.2)]
            beta = []
            
            for b_i in range(d):            
                b_i_sample = random.normalvariate(n_mu,n_sigma)
                r_unif = random.uniform(0,1)                
                if (r_unif < 1-sparse and b_i > 0):
                    b_i_sample = 0 # random.normalvariate(0,0.0001)
                beta.append(b_i_sample)
            print beta   
            dists.append(mixture.ProductDistribution([mixtureLinearGaussian.LinearGaussianDistribution(d, beta, sigma)]))
            lists.append(dists[j].sampleSet(sc[j]))
            notes = notes + [j]*sc[j]
            listdata = listdata + lists[j]
        
        data = mixture.DataSet()
        data.fromList(listdata,row_headers=notes)
        #data.center(columns=[0])
        data.writeFile(output + "/output_" + str(n+1) + ".txt")
        
        gen = mixture.MixtureModel(c,[1. / c] * c,dists,identifiable=0)
        
        file = open(output + "/model_" + str(n+1) + ".txt", "w")        
        file.write("Total Clusters: " + str(c) + "\n")
        file.write("Dimension: " + str(d) + "\n")
        file.write("Samples: " + str(s) + "\n\n\n")    
        for j in range(c):
            file.write("Component: " + str(j+1) + "\tTotal samples: " + str(sc[j]) + "\n")
            file.write(str(dists[j]))
            file.write("\n")
        
        print '\tgenerate output_' + str(n+1) + '.txt model_' + str(n+1) + '.txt in ' + output

if __name__ == "__main__":
    random.seed(231234)
    main()
