import numpy as np
import matplotlib.pyplot as plt
from pylab import *
import matplotlib.cm as cm
import string as s
import scipy.cluster.hierarchy as hierarchy
import scipy.spatial.distance as distance


#Le o arquivo com extensao ".res" e retorna duas listas: uma com os labels (header) e uma com a matriz das regressoes (matriz "B")
def read(clusterFile):
        f = open(clusterFile, 'r')

        # Faz a leitura do arquivo e salva em "texto"
        texto = []
        for line in f:   ## iterates over the lines of the file
                texto.append(line)
        f.close()
        texto.pop() #A ultima linha contem apenas '\n'

        # A primeira linha contem os labels. Desconsideramos as duas primeiras palavras ("ID" e "NT") e a ultima ("Sigma")
        header = texto.pop(0).split('\t')
        #print 'HeaderAntigo', header

        #Obtidos os headers, agora pegamos os coeficientes das regressoes
        matrizRegressao = []
        listaMedias = [] # Vai guardar os valores da coluna "b0" do arquivo .res
        count = 0
        for linha in texto:
                #O primeiro elemento de toda linha eh o numero de componentes
                #O segundo eh o b0
                #E o ultimo elemento eh o "Sigma"
                linhaParticionada = linha.split('\t')
                listaMedias.append(s.atof(linhaParticionada[1]))
#                matrizRegressao.append(linhaParticionada[1:-1])
                matrizRegressao.append(linhaParticionada[2:-1])
#                print linhaParticionada[2:-1] + [linhaParticionada[1]]
                for i in range(len(matrizRegressao[count])):
                        matrizRegressao[count][i] = s.atof(matrizRegressao[count][i])#transforma a string em numero
                count = count + 1

#        print texto
#        print header
#        print matrizRegressao
#        print listaMedias
        return header[2:-1], listaMedias, matrizRegressao

def maxTupla(tupla1, tupla2):
        return max(tupla[0], tupla[1])

#Recebe a matriz dos dados e um inteiro "top". Se a matriz tiver mais que "top" linhas, retorna apenas "top" linhas
#Caso contrario, retorna a matriz por completa
def filtrarMaiores(matrizRegressao, top, reverso = True):
        listaTuplaMaiores = []
        if top >= len(matrizRegressao):
            return range(len(matrizRegressao))
        else:
            for i in range(len(matrizRegressao)):
                    listaTuplaMaiores.append((np.max(np.absolute(matrizRegressao[i])), i))
            #print '\t TOP ->', top
            #print '\tLISTA TUPLA MAIORES', listaTuplaMaiores
            #listaOrdenada = sorted(listaTuplaMaiores, key = maxTupla)
            #return listaOrdenada
            listaTuplaMaiores.sort(reverse = reverso)
            #print '\tLISTA TUPLA MAIORES ORDENADA', listaTuplaMaiores
            quantidade = min(top, len(matrizRegressao))
            retorno = [ x[1] for x in listaTuplaMaiores[0:quantidade]]#Retorna uma lista com os indices dos que passaram no filtro
            return retorno
        

def filtrarDados(data, rowLabels, filtro):
        dadosFiltrados = []
        rowLabelsFiltrados = []
        for i in filtro:
                dadosFiltrados.append(data[i])
                rowLabelsFiltrados.append(rowLabels[i])
        return dadosFiltrados, rowLabelsFiltrados


#Recebe a matriz de regressao e o seu maior valor absoluto e retorna uma lista com as cores RGB
def toRGB(matriz, maiorPositivo, menorNegativo):
        colorRGB = []
        for indexLinha in range(len(matriz)):
                colorRGB.append([])
                for coluna in matriz[indexLinha]:
                        if coluna > 0:#Valor positivo -> COR VERMELHA
                                colorRGB[indexLinha].append((abs(coluna/maiorPositivo), 0, 0))
                        else:#Valor negativo -> COR VERDE
                                colorRGB[indexLinha].append((0, abs(coluna/menorNegativo), 0))
        return colorRGB

#Recebe a lista contendo toda as medias (coluna b0) das regressoes e retorna uma lista com as cores RGB
def toRGBMedias(listaMedias, maiorPositivo, menorNegativo):
        colorRGB = []
        colorRGB.append([])
        for media in listaMedias:
                if media > 0:#Valor positivo -> COR VERMELHA
                        colorRGB[0].append((abs(media/maiorPositivo), 0, 0))
                else:#Valor negativo -> COR VERDE
                        colorRGB[0].append((0, abs(media/menorNegativo), 0))
        return colorRGB

def modulo(x):
    if x < 0:
        return -x
    else:
        return x

        
def maiorModulo(matriz):
        #maxAbsoluto = 0
        maiorPositivo = -1000000
        menorNegativo = 1000000
        for linha in matriz:
                for coluna in linha:
                        #maxAbsoluto = max(maxAbsoluto, abs(coluna))
                        maiorPositivo = max(maiorPositivo, coluna)
                        menorNegativo = min(menorNegativo, coluna)
        return maiorPositivo, menorNegativo

#Insere espacos vazios numa lista de forma alternada.
#Exemplo: ['ab', 'cd', 'fg']->['ab', '', 'cd', '', 'fg']
def inserirBrancos(listaString):
        listaAlternada = []
        for i in listaString:
                listaAlternada.append('')
                listaAlternada.append(i)
        return listaAlternada
        
def computarRowLabels(labelsHistonas):
    size = len(labelsHistonas)
    if size == 1:
        return labelsHistonas
    elif size >= 2 and size <= 4:
        return inserirBrancos(labelsHistonas)
    else:
        return [''] + labelsHistonas

def computarColumnLabels(labelsComponents):
    size = len(labelsComponents)
    if size == 1:
        return ['','','',labelsComponents[0]]
    elif size >= 2 and size <= 4:
        return inserirBrancos(labelsComponents)
    else:
        return [''] + labelsComponents


#Parametro "cluster" pode apresentar os seguintes valores:
#       1 -> Faz o hierarquico apenas das linhas da matriz "data"
#       2 -> Faz o hierarquico apenas das colunas da matriz "data"
#       3 -> Faz o hierarquico das linhas e das colunas da matriz "data"
def clustergram(data, medias, rowLabels, columnLabels, outputPathPng, outputPathPdf, cluster = 3, rowPDist = 'euclidean', columnPDist = 'euclidean', method = 'single', top = 10):
        #Referencias -> Hierarquico: http://docs.scipy.org/doc/scipy/reference/cluster.hierarchy.html
        #               Distancias: http://docs.scipy.org/doc/scipy/reference/generated/scipy.spatial.distance.pdist.html
        #Hierarquico

        dadosTranspostos = np.transpose(data)
        #print '\tDADOS: ', data
        #Faz o filtro dos dados
        filtro = filtrarMaiores(dadosTranspostos, top)
        #print 'Filtrado ', filtro
        #print '\tRow Labels A ', rowLabels
        #Selecionar os dados filtrados de "data"
        #print '\Row Labels original ', rowLabels
        data, rowLabels = filtrarDados(dadosTranspostos, rowLabels, filtro)
        data = np.transpose(data)
        #print data, 'DadosFiltrados'
        #print '\tRow Labels B ', rowLabels
        
        ordemHierarquicoLinhas = range(len(data))
        ordemHierarquicoColunas = range(len(data[0]))

        if cluster != 2 and len(data) > 1:#Ou seja, se cluster == 1 ou cluster == 3 -> Faz o cluster das linhas
                S_linhas = distance.pdist(data, metric = rowPDist)
                #print data
                hierarquicoLinhas = hierarchy.linkage(S_linhas)
                ordemHierarquicoLinhas = hierarchy.leaves_list(hierarquicoLinhas)
                ##ordemHierarquicoLinhas.append(len(data))#Junta a linha com as medias (b0) na ultima posicao
                ##ordemHierarquicoLinhas = np.insert(ordemHierarquicoLinhas, len(data) - 1, len(data) - 1)#Junta a linha com as medias (b0) na ultima posicao
                ordemHierarquicoLinhas = np.argsort(medias)
                
        if cluster != 1 and len(data[0]) > 1:
                dadosSemMedias = np.transpose(data)#[0:-1]#Como a ultima linha dos dados eh o vetor de medias (b0), ele nao entra no cluster
                S_colunas = distance.pdist(dadosSemMedias, metric = columnPDist)
                hierarquicoColunas = hierarchy.linkage(S_colunas)
                ordemHierarquicoColunas = hierarchy.leaves_list(hierarquicoColunas)

        #Reordena "data" de acordo com o resultado do clusterHierarquico
        dadosHierarquicos = []
        columnLabelsHierarquicos = []
        for i in range(len(data)):
                #print i, ordemHierarquicoLinhas[i]
                dadosHierarquicos.append(data[ordemHierarquicoLinhas[i]])
                #rowLabelsHierarquicos.append(rowLabels[ordemHierarquicoLinhas[i]])
                columnLabelsHierarquicos.append(columnLabels[ordemHierarquicoLinhas[i]])

        dadosOrdenados = []
        columnLabelsHierarquicos = []
        rowLabelsHierarquicos = []
        for i in range(len(data)):
                dadosOrdenados.append([])
                columnLabelsHierarquicos.append(columnLabels[ordemHierarquicoLinhas[i]])
                for j in range(len(data[0])):
                        dadosOrdenados[i].append(data[ordemHierarquicoLinhas[i]][ordemHierarquicoColunas[j]])
                dadosOrdenados[i].append(medias[ordemHierarquicoLinhas[i]])#Adiciona a media da componente correspondente
        for i in range(len(data[0])):
            rowLabelsHierarquicos.append(rowLabels[ordemHierarquicoColunas[i]])
        rowLabelsHierarquicos.append('b0')#adiciona o label das medias (as medias aparecem na ultima linha)
        
        #print '\tROW LABELS HIERARQUICOS: ', rowLabelsHierarquicos

        #Obtem o maior valor positivo e negativo absoluto da matriz
        maxPositivoRegressao, minNegativoRegressao = maiorModulo(data)

        #Obtem os valores da matriz transformados em cores RGB----------------------------------------------------------------------------------------
        #O correto seria dadosOrdenados?
        matrizRGB = toRGB(np.transpose(dadosOrdenados), maxPositivoRegressao, minNegativoRegressao)

        #print 'DadosOrdenados', dadosOrdenados

        #Pinta a matriz na tela
        numHistonasExibidas = min(top, len(dadosTranspostos))
        numComponentes = len(columnLabelsHierarquicos)
        largura = 8
        if numComponentes > 1:
            largura = 4*((numComponentes+6)/4)
        fig = plt.figure(figsize=(largura, 6))#O tamanho default eh (8,6)
        ax = fig.add_subplot(111)
        im = ax.imshow(matrizRGB, aspect = 'auto', interpolation='nearest', shape = (numComponentes, numHistonasExibidas+1))
        xlabel('COMPONENT')
        #print 'Row Labels', rowLabel
        plt.yticks(range(len(rowLabelsHierarquicos)),rowLabelsHierarquicos, verticalalignment = 'center')
        ax.set_xticklabels(computarColumnLabels(columnLabelsHierarquicos), verticalalignment = 'top')
        #plt.colorbar(im, orientation='horizontal', shrink=0.8)
        plt.savefig(outputPathPng)
        plt.savefig(outputPathPdf)
        
        plt.close()
        
        return data, filtro

        #plt.show()

def boxPlot(data, outputPathPng, outputPathPdf, Ylabel):
    fig = plt.figure(figsize=(6,6))
    ax = fig.add_subplot(111)
    pos = np.array(range(len(data)))+1
    print len(data), len(data[0])
    bp = ax.boxplot( np.transpose(data), positions = pos, sym='gD', notch=1)
    
    ax.set_xlabel('COMPONENTS')
    ax.set_ylabel(Ylabel)
    plt.setp(bp['whiskers'], color='k',  linestyle='-' )
    plt.setp(bp['fliers'], markersize=3.0)
    fig.subplots_adjust(right=0.99,top=0.99)
    
    plt.savefig(outputPathPng)
    plt.savefig(outputPathPdf)
    
    #plt.show()
            

def paint(fileName):
        header, medias, matriz = read(fileName)

        #print matriz, 'matriz'
        #print medias, 'medias'
        
        rowLabels = header
        #print 'RowLabels', rowLabels
        columnLabels = range(1, len(matriz)+1)
        clustergram(matriz, medias, rowLabels, columnLabels)

       
        #plt.colorbar(im, orientation='horizontal', shrink=0.8)
        plt.savefig(fileName.replace('.res', '.pdf'))

        plt.show()

def main():
        if len(sys.argv) == 2:
                paint(sys.argv[1])
        else:
                print 'Eh preciso passar um arquivo .res como entrada'


if __name__ == '__main__':
  main()
