import mixture
import numpy
import random
import copy
import math
import mixtureLinearGaussian
from numpy.linalg import linalg as la
import scipy.stats

scikits=1

if scikits:
  from scikits.learn.linear_model import Lasso, ElasticNet
else:
  from glmnet.glmnet import elastic_net

  class ElasticNet(object):
        """ElasticNet based on GLMNET"""
        def __init__(self, alpha, rho=0.2, fit_intercept=True):
            super(ElasticNet, self).__init__()
            self.alpha = alpha
            self.fitIntercept=fit_intercept
            self.rho = rho
            self.coef_ = None
            self.rsquared_ = None

        def fit(self, X, y, weights=[],max_iter=1000000,tol=0.1, coef_init=[]):
            #print w
            n_lambdas, intercept_, coef_, _, _, rsquared_, lambdas, _, jerr \
            = elastic_net(X, y, self.rho, lambdas=[self.alpha],weights=weights,threshold=tol,standardize=False)
            # elastic_net will fire exception instead
            # assert jerr == 0
            #print n_lambdas, intercept_,rsquared_
            self.coef_ = coef_
            self.intercept_ = intercept_
            self.rsquared_ = rsquared_
            #print self.rsquared_
            return self

        def predict(self, X):
            return np.dot(X, self.coef_) + self.intercept_

        def __str__(self):
            n_non_zeros = (np.abs(self.coef_) != 0).sum()
            return ("%s with %d non-zero coefficients (%.2f%%)\n" + \
                    " * Intercept = %.7f, Lambda = %.7f\n" + \
                    " * Training r^2: %.4f") % \
                    (self.__class__.__name__, n_non_zeros,
                     n_non_zeros / float(len(self.coef_)) * 100,
                 self.intercept_[0], self.alpha, self.rsquared_[0])



class ElasticNetLinearDistribution(mixtureLinearGaussian.LinearGaussianDistribution):
	"""
	Linear Gaussian Distribution
	$0.5*||y-Xw||^2_2 \aplha \rho ||w||_1 + \alpha (1 - \rho) 0.5 ||w||_2^2
	"""

	def __init__(self, p, beta, sigma, alpha, rho):
		"""
		Constructor

		@param p: dimensionality of the distribution
		@param beta: coeficiente
		@param sigma: desvio padrao do erro
                @param lambd : lambd parameter for elastic net
		@param alpha : parameter for elastic net (normally called beta)
		"""
		assert len(beta) == p, len(sigma) == 1
		self.p = p
		self.suff_p = p
		self.beta = numpy.array(beta, dtype='Float64')		# create a array (numpy) for variable beta
		self.sigma = numpy.array(sigma, dtype='Float64')	# create a array (numpy) for variable sigma
		self.freeParams = p + 2
                self.predicted = []
                self.alpha = alpha 
		self.rho = rho
		self.mean = 0
                self.elastic= ElasticNet(alpha=alpha,rho=rho,fit_intercept=True)
		self.noise=0

	def __copy__(self):
		return ElasticNetLinearDistribution(self.p,self.beta, self.sigma, self.lambd)

	def __str__(self):
		return "ElasticNetLinearLasson:  ["+str(self.beta)+", "+str(self.sigma) +", "+str(self.alpha)+", "+str(self.rho)+"]"

	def __eq__(self,other):
		if not isinstance(other,ElasticNetLinearDistribution):
			return False
		if self.p != other.p:
			return False
		if not numpy.allclose( self.beta, other.beta ) or not numpy.allclose(self.sigma, other.sigma)  or not numpy.allclose(self.lambd, other.lambd):        
			return False
		return True

##        def predict(self,posterior,data):
## 		if isinstance(data, mixture.DataSet):
##			dt = data.internalData
##		elif isinstance(data,numpy.ndarray):
##			dt = data
##		else:
##			raise TypeError, "Unknown/Invalid input to MStep."

##		# Matrix column of 1's concatenated with rest of columns of data set of the matrix
##		x = numpy.concatenate((numpy.array([numpy.ones(len(dt))]).T, dt[:,1:]), axis=1)

##                return numpy.multiply(posterior,numpy.dot(x, self.beta.T))
##                #return (posterior,numpy.dot(x, self.beta.T))
                
                

	def isValid(self,x):
		if not len(x) == self.p:
			raise InvalidDistributionInput, "\n\tInvalid data: wrong dimension(s) "+str(len(x))+" in MultiNormalDistribution(p="+str(self.p)+")."
		for v in x:
			try:
				float(v)
			except (ValueError):
				raise InvalidDistributionInput, "\n\tInvalid data: "+str(x)+" in MultiNormalDistribution."

	def flatStr(self, offset):
		offset +=1
		return "\t"*offset+";ElasticNetRegression;"+str(self.p)+";"+str(self.beta.tolist())+";"+str(self.sigma.tolist())+"\n"

        def getSparsity(self):
            return len(numpy.flatnonzero(self.beta))



class ElasticNetPriorDistribution(mixture.PriorDistribution):
    """ Elastic Net (and lasso) prior distribution """

    def __init__(self, alpha, rho, a, b, fixed = 1):
        self.alpha = alpha
	self.rho = rho
	self.a = a 
	self.b = b
	self.factor = [1]*len(alpha)
        self.freeParams = len(alpha) + len(rho)
        self.constant_hyperparams = 1  # hyperparameters are constant
	self.fixed = fixed
	self.G = len(alpha)

      
    def pdf(self, models):
        prior = 0
        for i,m in enumerate(models):
          alphaaux = max(self.alpha[i],0)
          rhoaux =  max(self.rho[i],0.0)
	  factoraux = max(self.factor[i],0.00000000001)
          if self.alpha[i] > 0:
            if self.rho[i] < 1:
              prior +=  sum(numpy.log(scipy.stats.norm.pdf(m.beta[1:],loc=0,scale=1/numpy.sqrt(factoraux*alphaaux*(1-rhoaux)))))
            if self.rho[i] > 0:
              prior += sum(numpy.log(scipy.stats.laplace.pdf(m.beta[1:],loc=0,scale=1/(factoraux*alphaaux*rhoaux))))
	    #prior += max(0.0000000001,numpy.log(scipy.stats.gamma.pdf(1/(m.sigma[0]*m.sigma[0]),self.a,scale=self.b)))
     
        if numpy.isnan(prior):
            print ' aqui' 
            print "prior is nan"
            return -10000000.0
        else:
          return numpy.log(0.000000000000000001) # prior


    def isValid(self,x):
        if isinstance(x, ElasticNetPriorDistribution):
            return True
        else:
            return False
        
    def __copy__(self):        
        return ElasticNetPriorDistribution(self.alpha,self.rho,self.a,self.b,fixed=self.fixed)
    
    def __str__(self):
        return "ElasticNetPriorDistribution: "+str(self.alpha)+" "+str(self.rho)+" "+str(self.factor)  
    
    def updateHyperparameters(self, dists, posterior, data): 
        """
        Update the hyperparameters in an empirical Bayes fashion.
        
        @param dists: list of ProbabilityDistribution objects
        @param posterior: numpy matrix of component membership posteriors
        @param data: DataSet object
        """
        pass

    def posterior(self,m,x):  
        raise NotImplementedError, "Needs implementation"

    def marginal(self,x):
        raise NotImplementedError, "Needs implementation"

    def mapMStep(self, dist, posterior, data, mix_pi=None, dist_ind = None):
    
        """ Elastic Net XXX - to do MAP2 estimates
        """

	if isinstance(data, mixture.DataSet):
		dt = data.internalData
	elif isinstance(data,numpy.ndarray):
		dt = data
	else:
		raise TypeError, "Unknown/Invalid input to MStep."

	# First column of data set of the matrix
	y = dt[:,0]
	# Matrix column of 1's concatenated with rest of columns of data set of the matrix
	x = dt[:,1:]

        postsum = posterior.sum()

        #if scikits: 
  	posteriorsqrt = numpy.sqrt(posterior)
	xaux2 = numpy.array(numpy.multiply(x,numpy.matrix(posteriorsqrt).T))
  	yaux2 = numpy.multiply(y,posteriorsqrt)
        xaux = numpy.array(numpy.multiply(x,numpy.matrix(posterior).T))
        yaux = numpy.multiply(y,posterior)	

	if self.fixed != 1:	  
	  self.gamma = dist.p
	  dist.alpha = 2*self.gamma/(sum(abs(dist.beta)))
	  if dist.rho > 0:
    	    dist.alpha = self.gamma/((numpy.dot(dist.beta,dist.beta.T)))
	    dist.elastic.rho = dist.rho	    
	  dist.elastic.alpha = dist.alpha

	# making the beta proportional to group size and std
        self.factor[dist_ind] = postsum/(len(posterior)) #*(dist.sigma[0]**2))

        #setting parameters
	dist.elastic.alpha = dist.alpha*postsum/len(posterior)
	dist.elastic.rho = dist.rho
	self.rho[dist_ind]= dist.rho
	self.alpha[dist_ind]=dist.alpha
	
	# Beta estimation

	betashort = dist.beta[1:]
        if scikits:
            dist.elastic.fit(xaux2,yaux2-numpy.mean(yaux2),max_iter=1000000, coef_init=betashort,fit_intercept=True)
            betashort = dist.elastic.coef_.copy()
            dist.elastic.intercept_+=numpy.mean(yaux2)
            
            #print dist.elastic.intercept_
            dist.beta =  numpy.concatenate(([dist.elastic.intercept_],betashort),axis=1)
        else:    
            dist.elastic.fit(x,y,max_iter=1000000, coef_init=betashort,weights=posterior)
            betashort = dist.elastic.coef_.copy()
            betashort =  betashort[:,0]
            dist.elastic.intercept_+=numpy.mean(yaux2)
            dist.beta =  numpy.concatenate((dist.elastic.intercept_,betashort),axis=1)
        
	# Sigma estimation
	dist.predicted =  dist.elastic.intercept_ + numpy.dot(x, betashort.T)  
	y_x_betat = numpy.subtract(y, dist.predicted)
        dist.predicted = numpy.multiply(dist.predicted,posterior)
	
	sigma_numerator = numpy.dot(numpy.multiply(y_x_betat,posterior), y_x_betat) #+ 2*self.b
         # + 2*dist.alpha*dist.rho*sum(abs(dist.beta[1:])) + dist.alpha*(1-dist.rho)*numpy.dot(dist.beta[1:],dist.beta[1:].T) 
	sigma_denominator = posterior.sum() # + 2*self.a + 3*dist.p - 2
	
	dist.sigma[0] = max(0.00000001,numpy.sqrt( sigma_numerator/ sigma_denominator))
	
        dist.currentPosterior = posterior
        dist.freeParams = 2 + (dist.getSparsity())
        #print dist.freeParams

	#print "model", dist.mean, dist.sigma, sum(abs(dist.beta) > 0), postsum, dist.lambd, dist.alpha
	
    
	

