
import mixture
import mixtureLinearGaussian
import numpy
import random
import copy
import getopt
import os
import sys
import math
import matplotlib
matplotlib.use("Agg")
import pylab
#import markup
import shutil
import mixtureGLMNet
import optparse
from time import clock
import markup
#Graficos: hierarquico.py e graficoExpressaoOrdem
import hierarquico
import graficoExpressaoOrdem

def runMixture(minCluster,maxCluster,data,dist,repetitions,iterations,stopCriteria,modeltype,folds,exp_conf):
    """
    Function that run the mixture model for a range of clusters

    @param minCluster: cluster minimal number
    @param maxCluster: cluster maximum number
    @param data: DataSet object
    @param dist: ProductDistribution of one component
    @param repetitions: number of repetitions
    @param iterations: EM iterations
    @param stopCriteria: EM stop criteria
    @param bayesian: perform MAP estimation
    @param folds: folds for crossvalidation

    @return: models, NEC, BIC, AIC, classifies

        Example of run for th1 data with up to 3 models and 2 EM replications
    """
    models = []
    classifies = []
    correlations = []
    stds = []
    errors = []
    estds = []
    previousClusters = []
           
    for k in range(minCluster,maxCluster+1):
        print "#################### MIXTURE CLUSTER %d ####################" % k
        # mixture model for k components with distribution dist
        print 'aqui', modeltype
        train = mixtureComponents(k, dist, modeltype)
        # finding the best model
        correlation = 0
        std = 0
        error = 0
        estd = 0
        if folds > 1:
            [correlation, error] = crossvalidationregression(train,data,exp_conf,modeltype)
        [bestmix,previous] = estimateWithReplication(train,data, exp_conf ,modeltype)
        
        previousClusters.append(previous)

        models.append(bestmix)
        # classifying data
        classify = bestmix.classify(data,silent=1)
        classifies.append(classify)
        correlations.append(correlation)
        stds.append(numpy.std(correlation))
        errors.append(error)
        estds.append(numpy.std(error))

    for m in models: 
      m.updateFreeParams()
      print 'free', m.freeParams
    NEC,BIC,AIC = mixture.modelSelection(data,models)
    
    return models, NEC, BIC, AIC, classifies, correlations, stds, errors, estds, previousClusters

def runMixtureClassify(data, exp_conf,prior_type=1):
    """
    The function runs a classification for a mixture model
    
    @param data: DataSet object
    @param dist: distribution
    """
    n = data.noLabels #number of components
    pi = [1. / n] * n
    distributions = []
    for i in range(0, n):
        distributions.append(copy.deepcopy(exp_conf.dist))
    train = mixture.LabeledMixtureModel(n,pi,distributions)
    accuracy, mean_train_error, sum_matrix = crossvalidationclassification(data.N, data, train, exp_conf)
    return accuracy, mean_train_error, sum_matrix
        
def crossvalidationclassification(folds, data,  train, exp_conf):
    
    true_classify = 0
    false_classify = 0
    
    classify_lst = []
    coOcurrence_matrix_lst = [] 
    bestmix = None
    train_error = []
    
    cv = cv_samples(folds,data.N)
    
    coOcurrence_matrix_lst = []
    map_classify_lst = [] 

    #converting labels to numbers
    unique_classes = list(set(data.row_headers))
    classes = copy.deepcopy(data.row_headers)
    for c in range(0, len(classes)):
        for uc in range(0, len(unique_classes)):
            if classes[c] == unique_classes[uc]:
                classes[c] = uc
    
    #print data.row_headers
    #print classes
    count = -1

    for sample in range(0, data.N):
        #Train
        train = copy.deepcopy(data)
        train.removeSamples([train.sampleIDs[sample]])
        print "sample", sample, "removed"
        print train.row_headers
        
        train_classes = copy.deepcopy(classes)
        train_classes.pop(sample)
        print train_classes
        
        previous_posterior = None
        cm_argmax = None
        argmax = None
        
        model = mixtureComponents(1, exp_conf.dist, 'c')
        train.internalInit(model)
        classifies = []

	count +=1
	print "\nsample", count , "of", len(data.row_headers)-1, "samples\n"

        for k in range(exp_conf.min_cluster, exp_conf.max_cluster+1):
            print "#################### MIXTURE CLUSTER %d ####################" % k
            training = mixtureComponents(k, exp_conf.dist,  'c')
            bestmix, previous_posterior = estimateWithReplication(training,train,exp_conf, 'c')
            print "\nCLASSES", train_classes
            argmax=list(numpy.argmax(previous_posterior, axis=0))
            print "ARGMAX ", argmax
            cm_argmax, cm = confusionMatrixArgmax(data.noLabels, bestmix.G, train_classes, list(argmax))
            print "Confusion Matrix\n", cm
            print "ARGMAX - Confusion Matrix\n", cm_argmax
            true_train = 0
            false_train = 0
	    print len(argmax)
            for index in range(0, len(argmax)):
                if train_classes[index] == mapClassify(cm_argmax, argmax[index]):
                    true_train +=1
                else:
                    false_train +=1
                    print "False value: sample", index, "\n"
            training_error = 1-(true_train/float(false_train+true_train))
                
            train_error.append(training_error)
            print "\nTraining error", training_error, "%\n"
            
        #Test
        test = copy.deepcopy(data)
        test.removeSamples(train.sampleIDs)
        test.labels = []
        test.noLabels = 0
        classify = None
          
        test.internalInit(bestmix)
        p_post = numpy.zeros((bestmix.G,test.N),dtype='Float64')
        classify = bestmix.classify(test, 0, 10, p_post, 2) #test, prior_positive,prior_negative,previous_posterior,prior_type
        print "classify", classify
        mapped_classify = mapClassify(cm_argmax, classify[0])
        print "mapClassify", mapped_classify, "\n\n"
        argmax.insert(k, classify[0])

	cOccMatrix = data.getPairwiseConstraints(list(argmax), 1)
	print "cOccMatrix",cOccMatrix
        coOcurrence_matrix_lst.append(numpy.array(cOccMatrix))
        
	map_classify_lst.append(mapClassify(cm_argmax, classify[0]))
        if classes[sample] == mapClassify(cm_argmax, classify[0]):
        	true_classify +=1
                classify_lst.append(1)
        else:
                false_classify +=1
                classify_lst.append(0)
                print "False classify: sample", sample
            
    sum_matrix = numpy.zeros((len(classes),len(classes)), dtype='Float64') 
    for arr in coOcurrence_matrix_lst:
    	sum_matrix += arr
    print "\nFinal Co-Occurence", sum_matrix

    mean_train_error = sum(train_error)/float(len(train_error))
    print "\nMean Training Error",mean_train_error
    
    print "\nClasses", classes
    print "\nMapClassify", map_classify_lst

    accuracy = true_classify / float(true_classify+false_classify)
    print "\nLearning vector:", classify_lst, "\n"
    print "\nAccuracy", accuracy,"\n"
        
    return accuracy, mean_train_error, sum_matrix

def mapClassify(cm_argmax, number):
    return cm_argmax[number]

def confusionMatrixArgmax(noLabels, G, classes, argmax):
    
    cm = numpy.zeros((G, noLabels),dtype='Float64')
    for i in range(0, len(classes)):
        cm[argmax[i], classes[i]]+=1
    sum_cm = cm.sum(axis=0)
    cm_t = cm.transpose()
    print "Original cm \n", cm
    for index in range(0, len(sum_cm)):
        sum_cm[index] = 1/float(sum_cm[index])
        cm_t[index] = cm_t[index]*sum_cm[index]
    cm=cm_t.transpose()
    print "Sum Confusion Matrix ArgMax", sum_cm
    return list(numpy.argmax(cm, axis=1)), cm

def cv_samples(folds,n):
    foldSize = math.ceil(n/folds)
    rest = n%folds
    indices =range(n)
    random.shuffle(indices)
    results = []
    for i in range(folds):
      aux = []
      if i < rest:
        for j in range(foldSize+1):
           aux.append(indices.pop())
      else:
        for j in range(0,int(foldSize)):
           aux.append(indices.pop())
      aux.sort()
      results.append(aux)
    return results
    
def crossvalidationregression(mix,data,exp_conf,modeltype):
    """
    The function performs the N-fold cross validation
   
    @param mixture: training mixture
    

    """

    cv = cv_samples(exp_conf.folds,data.N)
    yfinal = numpy.zeros((data.N,1))
    all = []
    error = []
    for sample in cv:

        traindata = copy.copy(data)
        sampleAux = []
        for s in sample:
            sampleAux.append(data.sampleIDs[s])
        traindata.removeSamples(sampleAux)
            
        bextmix = None
        previous_posterior = None

        trainmix = copy.copy(mix)

        traindata.internalInit(trainmix)
        bestmix, previous = estimateWithReplication(trainmix,traindata, exp_conf ,modeltype)
        
        test = copy.copy(data)
        test.removeSamples(traindata.sampleIDs)
        test.internalInit(bestmix)
        sparse=[]
        if isinstance(trainmix.components[0],mixtureGLMNet.ElasticNetLinearDistribution):
          for m in trainmix.components:
              sparse.append(m.getSparsity())       
        [r,p,ypred,y,m,s,e,genes]=mixtureLinearGaussian.evaluateRegression(bestmix,test,train=traindata,type=0,sparse=sparse)


        #print "test", r,e, len(data), len(traindata), len(test)
        #print traindata.sampleIDs
        #print test.sampleIDs
        #print "correlation", r,p
        all.append(r)
        error.append(e)
    return [all,error]



def mixtureComponents(n,dist,modeltype):
    """
    Function that return a MixtureModel with n components of dist distribution
    
    @param n: number of components
    @param dist: ProductDistribution of one component
    
    @return: MixtureModel with n components of dist distribution
    """

    if modeltype == 'bayesianlinear':
      pipr = mixture.DirichletPrior(n,[1.0]*n)
      sp1 = mixtureLinearGaussian.LinearGaussianPriorDistribution([max(dist[0].alpha,0.001)]*n,fixed=0)
      prior = mixture.MixtureModelPrior(1.0,1.0, pipr,[sp1])
      pi = [1. / n] * n
      train = copy.copy(mixture.BayesMixtureModel(n,pi,[dist] * n,prior,identifiable=0))
    elif modeltype == 'elasticlinear':
      pipr = mixture.DirichletPrior(n,[1.01]*n)
      sp1 = mixtureGLMNet.ElasticNetPriorDistribution([dist[0].alpha]*n,[dist[0].rho]*n,0,0)
      prior = mixture.MixtureModelPrior(1.0,1.0, pipr,[sp1])
      pi = [1. / n] * n
      train = copy.copy(mixture.BayesMixtureModel(n,pi,[dist] * n,prior,identifiable=0))
    elif modeltype == 'lassolinear':
      pipr = mixture.DirichletPrior(n,[1.0]*n)
      sp1 =mixtureGLMNet.ElasticNetPriorDistribution([dist[0].alpha]*n,[1]*n,0,0)
      prior = mixture.MixtureModelPrior(1.0,1.0, pipr,[sp1])
      pi = [1. / n] * n
      train = copy.copy(mixture.BayesMixtureModel(n,pi,[dist] * n,prior,identifiable=0))
    elif modeltype == 'shrunkennormal':
      pipr = mixture.DirichletPrior(n,[1.0]*n)
      # extrair os valores medios dos dados m
      m = numpy.zeros((1,dist.p))
      s = 2 # deveria ser um argumento de entrada
      sps = [] 
      for i in range(dist.p):
          sps.append(mixtureShrunken.GausianLaplacePriorDistribution(m[0],s)) # melhorar os argumentos
          prior = mixture.MixtureModelPrior(1.0,1.0, pipr, sps)
      pi = [1. / n] * n
      train = copy.copy(mixture.BayesMixtureModel(n,pi,[dist] * n,prior,identifiable=0))
    elif modeltype == 'constrainednormal':
        pi = [1. / n] * n
        distributions = []
        for i in range(0, n):
            distributions.append(copy.deepcopy(dist))
        #print "entrou aqui"
        train = mixture.ConstrainedMixtureModel(n,pi,distributions)
    else:
      pi = [1. / n] * n	  
      train = copy.copy(mixture.MixtureModel(n,pi,[dist] * n,identifiable=0))
    train.err_tol=0.2  
    return train

def estimateWithReplication(_mixture, data, exp_conf ,modeltype):
    """
    Function replicating em estimation and returning maximum likelihood replicate
    
    @param _mixture: MixtureModel object
    @param data: DataSet object
    @param repetitions: number of repetitions
    @param iterations: EM iterations
    @param stopCriteria: EM stop criteria

    @return: mixture best
    """
    mixtureBest = []
    previous_posterior = None
    max = -float('inf');
    for j in range(exp_conf.repetitions):
        try:
            if modeltype == 'constrainednormal':
                maux = copy.copy(_mixture)
                previous_posterior = maux.modelInitialization(data, 0, 100, 2)
                #print "PI: ", maux.pi
                #print previous_posterior[0]
                #print data.N, data.p, data.labels
                [l,log_l] = maux.EM(data,exp_conf.iterations,exp_conf.stop_criteria, 0, 100, previous_posterior,  2)
                if log_l > max:
                	l_max = l
                	max = log_l
                	mixtureBest = maux
            elif modeltype in ['bayesianlinear','lassolinear','shrunkennormal','elasticlinear']:
                maux = copy.copy(_mixture)
                maux.modelInitialization(data)
                maux.mapEM(data,exp_conf.iterations,exp_conf.stop_criteria,silent=0)
                [l,log_l] = maux.EStep(data)
                if log_l > max:
                	l_max = l    
                	max = log_l
                	mixtureBest = maux
            else:
                maux = copy.copy(_mixture)
                maux.modelInitialization(data)
                maux.EM(data,exp_conf.iterations,exp_conf.stop_criteria,silent=0)
                [l,log_l] = maux.EStep(data)
            	if log_l > max:
                	l_max = l    
                	max = log_l
                	mixtureBest = maux
        except mixture.ConvergenceFailureEM:
            print maux
            pass
        #except mixtureLinearGaussian.EmptyComponent:
            #print "Empty Component"
    if mixtureBest == []:
        raise mixture.ConvergenceFailureEM,"Convergence failed."
    return mixtureBest, l_max

def usage():
    """
    Function that print usages informations for this program.
    """
    print """
Usage: mixtureTools.py [options] <input_data_file.txt>
Options:
    -h, --help			Print this help message
    -m, --max-cluster		Cluster maximum number
    -r, --repetitions		Number of repetitions (default 15)
    -i, --iterations		EM iterations (default 10)
    -s, --stop-criteria		EM stop criteria (default 0.1)
    -b, --bayesian   		Bayesian estimates
    -l, --lasso      		Lasso regression alpha (default 0.0)
    -e, --elastic      		Elastic Net regression beta  (default 0.0)
    -f, --folds                     Folds for cross-validation (defaut 0)
    -o, --output			Path dir output (default res)
"""

def createComponents(data, exp_conf):
    p = data.p
    sigma = [1]
    beta = []
    dist = None
    modeltype = None
    for i in range(data.p):	
        beta.append(random.normalvariate(0,1))
    if exp_conf.method == 0:
        p = []
        for i in range(data.p):
            p.append(mixture.NormalDistribution(0,1))
        dist = mixture.ProductDistribution(p)
        modeltype = "normal"
    elif exp_conf.method == 1:
	dist = mixture.ProductDistribution([mixtureLinearGaussian.LinearGaussianDistribution(p, beta, sigma)])
	modeltype = "linearnormal"
    elif exp_conf.method == 2:
	p = []
        for i in range(data.p):
            p.append(mixture.NormalDistribution(0,1))
        dist = mixture.ProductDistribution(p)
        modeltype = "constraintnormal"
    elif exp_conf.method == 3:
	dist = mixture.ProductDistribution([mixtureLinearGaussian.LinearGaussianDistribution(p, beta, sigma)])
	modeltype = "constraintlinearnormal"
    elif exp_conf.method == 4:
        if exp_conf.arg1 > 0:
	  dist = mixture.ProductDistribution([mixtureLinearGaussian.LinearGaussianDistribution(p, beta, sigma, alpha=exp_conf.arg1)])
        else:
	  dist = mixture.ProductDistribution([mixtureLinearGaussian.LinearGaussianDistribution(p, beta, sigma)])

	modeltype = "bayesianlinear"
    elif exp_conf.method == 5:
	dist = mixture.ProductDistribution([mixtureGLMNet.ElasticNetLinearDistribution(p, beta, sigma, exp_conf.arg1, 0)])
        print 'arg', exp_conf.arg1
	modeltype = "lassolinear"
    elif exp_conf.method == 6:
	dist = mixture.ProductDistribution([mixtureGLMNet.ElasticNetLinearDistribution(p, beta, sigma, exp_conf.arg1, exp_conf.arg2)])
        print 'arg', exp_conf.arg1, exp_conf.arg2
	modeltype = "elasticlinear"
    elif exp.conf.method == 7:
	p = []
        for i in range(data.p):
            p.append(mixture.NormalDistribution(0,1))
        dist = mixture.ProductDistribution(p)
	modeltype = "shrunkennormal"
#    elif exp_conf.method == 8:
#	dist = mixture.ProductDistribution([mixtureLinearGaussian.LinearGaussianDistribution(p, beta, sigma)])
#	modeltype = "shrunkenlinearnormal"
    else:
	return
#    if exp_conf.elastic > 0:
#        dist = mixture.ProductDistribution([mixtureGLMNet.ElasticNetLinearDistribution(p, beta, sigma, exp_conf.lasso*p, exp_conf.elastic*p)])
#        modeltype = 'e_'+str(elastic)+'_'+str(lasso)
#    elif exp_conf.lasso > 0:
#        modeltype = 'l_'+str(lasso)
#        dist = mixture.ProductDistribution([mixtureGLMNet.ElasticNetLinearDistribution(p, beta, sigma, exp_conf.lasso*p, 0)])
#    elif exp_conf.bayesian == True:
#        modeltype = 'b'
#        dist = mixture.ProductDistribution([mixtureLinearGaussian.LinearGaussianDistribution(p, beta, sigma)])
#    elif exp_conf.distribution == "normal":
#        p = []
#        for i in range(data.p):
#            p.append(mixture.NormalDistribution(0,1))
#            dist = mixture.ProductDistribution(p)
#        modeltype = "normal"
#    elif exp_conf.distribution == "linearnormal":
#        modeltype = 'g'
#        dist = mixture.ProductDistribution([mixtureLinearGaussian.LinearGaussianDistribution(p, beta, sigma)])
    # XXX - fix input codes for distinct types of distributions
    exp_conf.dist = dist
    return modeltype

def writeClusterReport(data, filename, output_path, bayesian,  modeltype, exp_conf,NEC, BIC, AIC, models,correlations, stds, errors, estds, previousClusters):
    

    clusters = range(exp_conf.min_cluster, exp_conf.max_cluster+1)
    bestmodel = BIC.index(min(BIC))

    #NEC = numpy.array(NEC)
    BIC = numpy.array(BIC)
    AIC = numpy.array(AIC)
    
    #pylab.plot(NEC, label="NEC")
    pylab.semilogy(clusters, BIC, label="BIC")
    pylab.ylim(min(BIC),max(BIC))
    
    #pylab.semilogy(clusters, AIC, label="AIC")
    
    try:
        os.stat(output_path)
    except:
        os.mkdir(output_path)
    try:
        os.stat(output_path + '/pdf')
    except:
        os.mkdir(output_path + '/pdf')
    try:
        os.stat(output_path + '/genes')
    except:
        os.mkdir(output_path + '/genes')
    try:
        os.stat(output_path + '/m')
    except:
        os.mkdir(output_path + '/m')
    try:
        os.stat(output_path + '/png')
    except:
        os.mkdir(output_path + '/png')
    
    pylab.xlabel('clusters')
    pylab.legend()
    pylab.savefig(output_path + "/pdf/" + filename[:-4] + "_" + modeltype + "_BIC_AIC.pdf")
    pylab.savefig(output_path + "/png/" + filename[:-4] + "_" + modeltype + "_BIC_AIC.png")
    pylab.hold(False)
    
    #if bestmodel == 0:
    #    data.setNotes([0] * data.N)
    #else: 
    #    data.setNotes(classifies[bestmodel-1])
    #data.writeFile(output_path + "/" + filename[:-4] + "_"+modeltype+"_classifies_" + str(clusters[bestmodel]) + ".res")

    headers = data.headers
    styles = ( 'styles/layout.css', 'styles/table.css' )


    for i,m in enumerate(models):

      #print m
      #train = copy.deepcopy(data)
      #test = copy.deepcopy(data)
      
      #ids = [train.sampleIDs[k] for k in range(0,len(train),2 )]
      #train.removeSamples(ids)
      #train.internalInit(m)
      #print len(ids), len(test)
      
      #ids = [test.sampleIDs[k] for k in range(1,len(test),2 )]
      #print len(ids), len(test)
      #test.removeSamples(ids)
      #test.internalInit(m)
      sparse=[]

      if isinstance(m.components[0],mixtureGLMNet.ElasticNetLinearDistribution):
          for comp in mcomponents:
              sparse.append(comp.getSparsity())       

      [r,p,predy,y,means,stdsaux,error,genes] = mixtureLinearGaussian.evaluateRegression(m,data,type=0)


      page_cluster = markup.page()
      page_cluster.init(title="Results " + filename[:-4] + '_' + str(clusters[i]) + "_" + modeltype + '_cluster', css=styles)
      page_cluster.h1("Results " + filename[:-4] + '_' + str(clusters[i]) + "_" + modeltype + '_cluster')
      #_res_
      res = open(output_path + "/" + filename[:-4] + '_' + str(clusters[i]) +"_"+modeltype+ '_cluster' + '.res', 'w')
      res.write('#comp\tb0')

      if exp_conf.folds == 1:
        correlations[i] = r
        stds[i] = 0.0
        errors[i] = error
        estds[i] = 0.0
        
      #_html_
      page_cluster.table.open(border=1)
      page_cluster.thead.open()
      page_cluster.th('Results')
      page_cluster.thead.close()
      page_cluster.tbody.open()
      page_cluster.tr.open()
      page_cluster.td("#comp")
      page_cluster.td("b0")

      for h in range(len(headers) - 1):
        #_html_
        page_cluster.td(str(headers[h+1]))
        #_res_
        res.write('\t' + str(headers[h+1]))
        
      #_html_
      page_cluster.td("sigma")
      page_cluster.tr.close()
      #_res_
      res.write('\tSigma\n')

      file = open(output_path + "/m/" + filename[:-4] + "_"+str(clusters[i])+"_"+modeltype+".m",'w')
      means = []
      regressionMatrix = []
      for j,c in enumerate(m.components):
          beta = c.distList[0].beta.tolist()
          file.write("x(:,"+str(j+1)+")="+str(beta)+'\n')
          
          #_html_
          page_cluster.tr.open()
          page_cluster.td(str(j+1))
          #_res_
          res.write(str(j+1))

          for k in range(len(beta)):
              #_html_
              page_cluster.td(str(beta[k]))
              #_res
              res.write('\t' + str(beta[k]))

          means.append(beta[0])
          regressionMatrix.append(beta[1:])

          #_html_
          page_cluster.td(str(c.distList[0].sigma[0]))
          page_cluster.tr.close()
          #_res_
          res.write('\t' + str(c.distList[0].sigma[0]))
          res.write('\n')

      topParam = 10
      graphicPathPng = output_path + "/png/" + filename[:-4] + '_' + str(i+1) + "_" + modeltype# + '_clustergram.png'
      graphicPathPdf = output_path + "/pdf/" + filename[:-4] + '_' + str(i+1) + "_" + modeltype# + '_clustergram.pdf'
      filteredData, filtro = hierarquico.clustergram(regressionMatrix, means, headers[1:], range(1, len(regressionMatrix)+1), graphicPathPng + '_clustergram.png', graphicPathPdf + '_clustergram.pdf', top = topParam)
      #graficoExpressaoOrdem.pintar(filename, regressionMatrix, means, previousClusters[j], headers[1:], output_path + "/png/", output_path + "/pdf/")
      links = graficoExpressaoOrdem.pintar(filename, filteredData, means, previousClusters[j - exp_conf.min_cluster + 1], headers[1:], output_path + "/png/", output_path + "/pdf/", top = topParam, filtro = filtro)
      outputPathPng = output_path + "/png/" + filename[:-4] + "_" + modeltype + "_boxPlot"
      outputPathPdf = output_path + "/pdf/" + filename[:-4] + "_" + modeltype + "_boxPlot"
      if exp_conf.folds<2:
          correlationsaux = numpy.array([correlations,correlations]).T
          errorsaux = numpy.array([errors,errors]).T
      else:
          correlationsaux = correlations
          errorsaux = errors
      hierarquico.boxPlot(correlationsaux, outputPathPng + "_correlations.png", outputPathPdf + "_correlations.pdf", "CORREALATIONS")
      hierarquico.boxPlot(errorsaux, outputPathPng + "_errors.png", outputPathPdf + "_errors.pdf", "ERRORS")
      
      #_html_
      page_cluster.tbody.close()
      page_cluster.table.close()
      #page_cluster.img(src = "png/" + filename[:-4] + "_" + modeltype + "_boxPlot_correlations.png")
      #page_cluster.img(src = "png/" + filename[:-4] + "_" + modeltype + "_boxPlot_errors.png")
      page_cluster.img(src="png/" + filename[:-4] + "_"+str(clusters[i])+"_"+modeltype+"_predicted.png")
      page_cluster.img(src = "png/" + filename[:-4] + '_' + str(clusters[i]) + "_" + modeltype + '_clustergram.png')
      for link in links:
        page_cluster.img(src=link)
      if i != 0:
        page_cluster.img(src="tif/" + filename[:-4] + "_tf_"+str(clusters[i])+"_"+modeltype+"_cluster.tif")
      page_cluster.h3("Genes")
      for j,gs in enumerate(genes):
        page_cluster.a("Cluster " + str(j+1), href="gene_cluster_" + str(clusters[i]) + "_" + str(j+1) + ".html")
      #_res_
      res.write('\n')
      res.close()
      file.close()

      #_html_file_
      page_cluster_f = open(output_path + "/result_cluster_" + str(clusters[i]) + ".html", "w")
      page_cluster_f.write(str(page_cluster))
      page_cluster_f.close()

      file = open(output_path + "/genes/" + filename[:-4] + "_"+str(clusters[i])+"_"+modeltype+".genes",'w')
      for j, gs in enumerate(genes):
          file.write(' '.join(gs)+'\n')
          #_html_file2
          page_cluster = markup.page()
          page_cluster.init(title="Genes " + filename[:-4] + '_' + str(clusters[i]) + "_" + modeltype + '_cluster_' + str(j+1), css=styles)
          page_cluster.h1("Genes " + filename[:-4] + '_' + str(clusters[i]) + "_" + modeltype + '_cluster_' + str(j+1))

          #_html_
          page_cluster.table.open(border=1)
          page_cluster.thead.open()
          page_cluster.th('Genes')
          page_cluster.a("G:Profiler",href="http://biit.cs.ut.ee/gprofiler/gconvert.cgi?organism=mmusculus&target=ENSG&output=html&query=+"+"+".join(gs))
          page_cluster.thead.close()
          page_cluster.tbody.open()
         
                        
                  
          for g in gs:
            page_cluster.tr.open()
            page_cluster.td.open()	  
            page_cluster.a(str(g), href="http://www.ncbi.nlm.nih.gov/entrez/query.fcgi?db=gene&dopt=summary&term=mouse[organism]+AND+"+str(g))
            page_cluster.td.close()
            page_cluster.tr.close()

          #_html_
          page_cluster.tbody.close()
          page_cluster.table.close()

          #_html_file_
          page_cluster_f = open(output_path + "/gene_cluster_" + str(clusters[i]) + "_" + str(j+1) + ".html", "w")
          page_cluster_f.write(str(page_cluster))
          page_cluster_f.close()
      file.close()

      #graficosSimples = graficoExpressaoOrdem.pintarSimples(filename, output_path + "/png/", output_path + "/pdf/")
      
      print "regression", r, p
      print "means", means
      print "std", stdsaux
      print "error", error

      pylab.figure()
      pylab.plot(predy,y,'.')
      pylab.xlabel('predicted expression')
      pylab.ylabel('expression')
      pylab.title(filename[:-4] + " #Comps: "+str(clusters[i])+" "+modeltype+" - corr "+str(r))
      pylab.savefig(output_path + "/pdf/" + filename[:-4] + "_"+str(clusters[i])+"_"+modeltype+"_predicted.pdf")
      pylab.savefig(output_path + "/png/" + filename[:-4] + "_"+str(clusters[i])+"_"+modeltype+"_predicted.png")
      
    print "\n"
    print "##################### Correlations ######################\n", correlations, "\n", stds, "\n", errors, "\n", estds	
    print "##################### NEC ######################\n", NEC, "\n"
    print "##################### BIC ######################\n", BIC, "\n"
    print "##################### AIC ######################\n", AIC, "\n"

    res = open(output_path + "/" + filename[:-4] +"_"+modeltype + '.res', 'w')
    res.write('Correlations')
    
    for i in range(len(correlations)):
        res.write('\t' + str(numpy.mean(correlations[i])))
    res.write('\n')
        
    res.write('Stds')
    for i in range(len(stds)):
        res.write('\t' + str(stds[i]))
    res.write('\n')
    
    res.write('Errors')
    for i in range(len(errors)):
        res.write('\t' + str(errors[i]))
    res.write('\n')

    res.write('Estds')
    for i in range(len(estds)):
        res.write('\t' + str(estds[i]))
    res.write('\n')

    res.write('NEC')
    for i in range(len(NEC)):
        res.write('\t' + str(NEC[i]))
    res.write('\n')

    res.write('BIC')
    for i in range(len(BIC)):
        res.write('\t' + str(BIC[i]))
    res.write('\n')

    res.write('AIC')
    for i in range(len(AIC)):
        res.write('\t' + str(AIC[i]))
    res.write('\n')

    res.close()
    
    page = markup.page()
    page.init(title="Results " + filename[:-4] + "_" + modeltype, css=styles)
    page.h1("Results " + filename[:-4] + "_" + modeltype)
    
    page.img(src="png/" + filename[:-4] + "_" + modeltype + "_BIC_AIC.png")
    
    page.img(src = "png/" + filename[:-4] + "_" + modeltype + "_boxPlot_correlations.png")
    page.img(src = "png/" + filename[:-4] + "_" + modeltype + "_boxPlot_errors.png")
        
    page.h3("Clusters Results")

    for i in clusters:
        page.a("Cluster " + str(i), href="result_cluster_" + str(i) + ".html")

    page.h2("Arguments")

    page.table.open(border=1)
    page.tbody.open()
    
    page.tr.open()
    page.td("max cluster")
    page.td(str(exp_conf.max_cluster), width="50", align="right")
    page.tr.close()
    
    page.tr.open()
    page.td("repetitions")
    page.td(str(exp_conf.repetitions), width="50", align="right")
    page.tr.close()

    page.tr.open()
    page.td("iterations")
    page.td(str(exp_conf.iterations), width="50", align="right")
    page.tr.close()

    page.tr.open()
    page.td("stop criteria")
    page.td(str(exp_conf.stop_criteria), width="50", align="right")
    page.tr.close()

    page.tr.open()
    page.td("Modeltype")
    page.td(modeltype, width="50", align="right")
    page.tr.close()

    page.tr.open()
    page.td("folds")
    page.td(str(exp_conf.folds), width="50", align="right")
    page.tr.close()

    page.tr.open()
    page.td("input data file")
    page.td(filename, width="50", align="right")
    page.tr.close()
    
    page.tbody.close()
    page.table.close()	

    #for figura in graficosSimples:
        #page.img(src = figura)
        
    index_file = open(output_path + "/index.html", "w")
    index_file.write(str(page))
    index_file.close()

    #shutil.copy("styles/layout.css", output_path + "/layout.css")
    #shutil.copy("styles/table.css", output_path + "/table.css")

class ExpConf:
	def __init__(self,min_cluster,max_cluster,repetitions,stop_criteria,folds,method,arg1,arg2,iterations):
		self.min_cluster = min_cluster
		self.max_cluster = max_cluster
		self.repetitions = repetitions
		self.stop_criteria = stop_criteria
		self.folds = folds
		self.method = method
		self.arg1 = arg1
		self.arg2 = arg2
		self.iterations = iterations
		self.dist = None


def main():

    # We need a usage descritption here
    # how to run python mixtureTools2.py -m 1 -p ../LymphoidRegulation/exp/mixtureregression/th1_mod.txt  --max-cluster 2

    #  -m 0 - Normal
    #     1   Constrained Normal
    #     2   Liner Model
    #     4   Bayesian
    #     5   Lasso (requires arg1)
    #     6   Elastic Net (requires arg1 e arg2)
    #     7   Shrunken (requires arg1)

    
    t0 = clock()
    usage = "Usage: %prog [options] arg1 arg2...\nTry %prog -h or %prog --help to see the available options"
    parser = optparse.OptionParser(usage)
    parser.add_option('-m', '--method', type="int", dest="method", default="0", help="Type of method  (0) Normal, (1) Constrained Normal (2) Liner Model, (4) Linear Bayesian (5) Linear Lasso (requires arg1) (6) Linear Elastic Net (requires arg1 e arg2) (7)   Shrunken (requires arg1)")
    parser.add_option('-f', '--folds', type="int",  dest="folds", default=1, help="Number of folds [default:%default]")
    parser.add_option('-i', '--iterations', type="int",  dest="iterations", default=10, help="EM iterations [default:%default]")
    parser.add_option('-o', '--output', type="string",  dest="output_path", default='res', help="Path of output file [default:%default]")
    parser.add_option('-p', '--path', type="string", dest="filename", default=None, help="Path to datafile")
    parser.add_option('-r', '--repetitions', type="int",  dest="repetitions", default=15, help="Number of repetitions [default:%default]")
    parser.add_option('-s', '--stop-criteria', type="float",  dest="stop_criteria", default=0.1, help="EM stop criteria [default:%default]")
    parser.add_option('--arg1', type="float", dest="arg1", default="0", help="First argument of the method")
    parser.add_option('--arg2', type="float", dest="arg2", default="0", help="Second argument of the method")	
    parser.add_option('--max-cluster', type="int",  dest="max_cluster", default=2, help="Cluster maximum number [default:%default]")
    parser.add_option('--min-cluster', type="int",  dest="min_cluster", default=2, help="Cluster maximum number [default:%default]")
    parser.add_option('--classify', action="store_true", dest="classify", default=False)
    parser.add_option('--log', action="store_true", dest="log_matrix", default=False, help="Log of each matrix value [default:%default")
    parser.add_option('--tranpose', action="store_true", dest="transpose", default=False, help="Tranpose matrix (individials as collumns)")
    options, args = parser.parse_args()

    #print 'options',options.arg1,options.arg2 

    
    exp_conf = ExpConf(options.min_cluster,options.max_cluster,options.repetitions,options.stop_criteria,options.folds,options.method,options.arg1,options.arg2, options.iterations)

    filename = options.filename
    output_path = options.output_path
    modeltype = None
    
    random.seed(3586662)

    data = None
    if (os.path.exists(filename)):
        print filename
        if options.method == 2:
          data = mixture.ConstrainedDataSet()
        else:
          data = mixture.DataSet()
        data.fromFile(filename, col_headers=True, row_headers=True)
        if(options.transpose):
          data.transpose()
        if(options.log_matrix):
          data.logTransform()
          print 'log of data' 
    else:
        print "Input data file does not exist."
        sys.exit(2)

    print data.dataMatrix[1]

    # creating a component
    modeltype = createComponents(data,exp_conf)
    
    # run the main funcition
    if options.classify:
        if exp_conf.method == 2:
            data.setConstrainedLabelsFromRowHeader()
            data.setPairwiseConstraints(None,   data.getPairwiseConstraints(data.row_headers, 2))
	    accuracy, mean_train_error, sum_matrix = runMixtureClassify(data,exp_conf,2)

	    #write co-coccurrence matrix 
	    newfilename = "_occurrenceMatrix"+str(options.max_cluster)+".txt"
            filename = filename.replace(".txt", newfilename)
            f = open(filename, "w")
	   
            numpy.savetxt(f, sum_matrix, fmt="%i", delimiter='\t')

	    tf = clock()
	    print "total execution time = ", tf-t0, "seconds"
    else:
        models, NEC, BIC, AIC, classifies, correlations, stds, errors, estds, previousClusters = runMixture(exp_conf.min_cluster, exp_conf.max_cluster, data, exp_conf.dist, exp_conf.repetitions, exp_conf.iterations, exp_conf.stop_criteria, modeltype, exp_conf.folds, exp_conf)
        writeClusterReport(data, filename, output_path, 0,  modeltype,exp_conf, NEC, BIC, AIC, models, correlations, stds, errors, estds, previousClusters)

if __name__ == "__main__":
    main()
